/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage;

import com.intellij.openapi.actionSystem.IdeActions;
import com.intellij.openapi.components.ApplicationComponent;
import net.intellij.plugins.lineage.actions.CreateClassAction;
import net.intellij.plugins.lineage.actions.HighlightUsagesAction;
import net.intellij.plugins.lineage.actions.OverridingAction;
import net.intellij.plugins.lineage.actions.PasteClassAction;
import net.intellij.plugins.lineage.actions.PasteFilesAction;
import net.intellij.plugins.lineage.tools.SubclassType;
import org.jetbrains.annotations.NotNull;

public class LineageApplicationComponent implements ApplicationComponent
{
	@NotNull
	public String getComponentName()
	{
		return "LineageApplicationComponent";
	}

	public void initComponent()
	{
		final OverridingAction highlightUsagesAction = new HighlightUsagesAction();
		highlightUsagesAction.override(IdeActions.ACTION_HIGHLIGHT_USAGES_IN_FILE);
		final OverridingAction createClassAction = new CreateClassAction(SubclassType.CLASS);
		createClassAction.override("NewClass");
		final OverridingAction createInterfaceAction = new CreateClassAction(SubclassType.INTERFACE);
		createInterfaceAction.override("NewInterface");
		final OverridingAction createEnumAction = new CreateClassAction(SubclassType.ENUM);
		createEnumAction.override("NewEnum");
		final OverridingAction createAnnotationAction = new CreateClassAction(SubclassType.ANNOTATION_TYPE);
		createAnnotationAction.override("NewAnnotationType");
		final OverridingAction pasteAction = new PasteClassAction();
		pasteAction.override(IdeActions.ACTION_PASTE);
//		final OverridingAction pasteFilesAction = new PasteFilesAction();
//		pasteFilesAction.override(IdeActions.ACTION_PASTE);
	}

	public void disposeComponent()
	{
		OverridingAction.restore(IdeActions.ACTION_HIGHLIGHT_USAGES_IN_FILE);
		OverridingAction.restore("NewClass");
		OverridingAction.restore("NewInterface");
		OverridingAction.restore("NewEnum");
		OverridingAction.restore("NewAnnotationType");
	}
}