/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage;

import com.intellij.codeInsight.intention.IntentionAction;
import com.intellij.codeInsight.intention.IntentionManager;
import com.intellij.codeInsight.intention.impl.CreateSubclassAction;
import com.intellij.openapi.components.ProjectComponent;
import net.intellij.plugins.lineage.intentions.SubClassIntention;
import net.intellij.plugins.lineage.tools.SubclassType;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.List;

public class LineageProjectComponent implements ProjectComponent
{
	public void projectOpened()
	{
		final IntentionManager intentionManager = IntentionManager.getInstance();
		final String[] category = new String[]{"Classes"};
		intentionManager.registerIntentionAndMetaData(new SubClassIntention(false, SubclassType.CLASS), category);
		intentionManager.registerIntentionAndMetaData(new SubClassIntention(true, SubclassType.CLASS), category);
		intentionManager.registerIntentionAndMetaData(new SubClassIntention(false, SubclassType.INTERFACE), category);
		intentionManager.registerIntentionAndMetaData(new SubClassIntention(true, SubclassType.INTERFACE), category);
		final Class<? extends IntentionManager> aClass = intentionManager.getClass();
		final Field[] fields = aClass.getDeclaredFields();
		outer:
		for (Field field : fields)
		{
			final Class<?> fieldType = field.getType();
			if ("java.util.List".equals(fieldType.getCanonicalName()))
			{
				try
				{
					field.setAccessible(true);
					final List<IntentionAction> intentionList =
							(List<IntentionAction>)field.get(intentionManager);
					for (Iterator<IntentionAction> it = intentionList.iterator(); it.hasNext();)
					{
						final IntentionAction intentionAction = it.next();
						if (intentionAction instanceof CreateSubclassAction)
						{
							it.remove();
							break outer;
						}
					}
				}
				catch (IllegalAccessException e)
				{
					throw new RuntimeException(e);
				}
			}
		}
	}

	public void projectClosed()
	{}

	@NotNull
	public String getComponentName()
	{
		return "LineageProjectComponent";
	}

	public void initComponent()
	{}

	public void disposeComponent()
	{}
}