/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.actions;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.command.CommandProcessor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.*;
import com.intellij.ide.IdeView;
import net.intellij.plugins.lineage.ui.SubClassDialog;
import net.intellij.plugins.lineage.tools.SubclassTools;
import net.intellij.plugins.lineage.tools.SubclassType;

/**
 * Because this action overrides pre-existing actions it is registered in
 * {@link net.intellij.plugins.lineage.LineageApplicationComponent}
 * @noinspection ComponentNotRegistered
 * */
public class CreateClassAction extends OverridingAction
{
	private final SubclassType subclassType;

	public CreateClassAction(SubclassType subclassType)
	{
		this.subclassType = subclassType;
	}

	protected boolean performAction(AnActionEvent event)
	{
		final PsiElement context = event.getData(DataKeys.PSI_ELEMENT);
		if (context == null)
		{
			return false;
		}
		final CommandProcessor commandProcessor = CommandProcessor.getInstance();
		final Project project = context.getProject();
		final SubclassType subclassType = this.subclassType;
		final SubClassDialog dialog =
				new SubClassDialog("", null, context, subclassType, false, true);
		dialog.show();
		if (!dialog.isOK())
		{
			return true;
		}
		final IdeView view = event.getData(DataKeys.IDE_VIEW);
		commandProcessor.executeCommand(project, new Runnable()
		{
			public void run()
			{

				final PsiClass newClass =
						SubclassTools.createTopLevelSubclass(dialog.getSubclassName(), null,
								dialog.getTargetDirectory(), subclassType, dialog.getVisibility(),
								dialog.shouldMakeAbstract());
				if (view !=  null && newClass != null)
				{
					final PsiJavaToken leftBrace = newClass.getLBrace();
					view.selectElement(leftBrace);
				}
			}
		}, "Create " + subclassType + " '" + dialog.getSubclassName() + '\'',
				"Create " + subclassType);
		return true;
	}
}