/*
 * Copyright 2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.actions;

import com.intellij.codeInsight.daemon.impl.HighlightInfo;
import com.intellij.codeInsight.daemon.impl.HighlightVisitor;
import com.intellij.codeInsight.daemon.impl.RefCountHolder;
import com.intellij.codeInsight.daemon.impl.analysis.HighlightInfoHolder;
import com.intellij.codeInsight.daemon.impl.quickfix.CreateClassFromNewFix;
import com.intellij.codeInsight.daemon.impl.quickfix.CreateClassFromUsageFix;
import com.intellij.codeInsight.daemon.impl.quickfix.CreateInnerClassFromUsageFix;
import com.intellij.codeInsight.intention.IntentionAction;
import com.intellij.openapi.util.Pair;
import com.intellij.openapi.util.TextRange;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import net.intellij.plugins.lineage.intentions.LineageCreateClassFromUsageFix;
import net.intellij.plugins.lineage.tools.SubclassType;

import java.util.Iterator;

public class LineageHighlightVisitor implements HighlightVisitor
{
	public boolean suitableForFile(PsiFile file)
	{
		return true;
	}

	public void visit(PsiElement element, HighlightInfoHolder infoHolder)
	{
		if (!infoHolder.hasErrorResults())
		{
			return;
		}
		for (final HighlightInfo highlightInfo : infoHolder)
		{
			if (highlightInfo.quickFixActionRanges != null)
			{
				boolean buildCreateClassFromUsageQuickFixes = false;
				boolean buildCreateClassFromNewQuickFixes = false;
				for (Iterator<Pair<HighlightInfo.IntentionActionDescriptor, TextRange>> iterator =
						highlightInfo.quickFixActionRanges.iterator(); iterator.hasNext();)
				{
					final Pair<HighlightInfo.IntentionActionDescriptor, TextRange> pair =
							iterator.next();
					final HighlightInfo.IntentionActionDescriptor actionDescriptor =
							pair.getFirst();
					final IntentionAction action = actionDescriptor.getAction();
					if (action instanceof CreateClassFromUsageFix)
					{
						buildCreateClassFromUsageQuickFixes = true;
						iterator.remove();
					}
					else if (action instanceof CreateInnerClassFromUsageFix)
					{
						buildCreateClassFromUsageQuickFixes = true;
						iterator.remove();
					}
					else if (action instanceof CreateClassFromNewFix)
					{
						buildCreateClassFromNewQuickFixes = true;
						buildCreateClassFromUsageQuickFixes = false;
						iterator.remove();
					}
				}
				if (buildCreateClassFromUsageQuickFixes)
				{
					final TextRange textRange = element.getTextRange();
					final HighlightInfo.IntentionActionDescriptor newDescriptor1 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.CLASS, false),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor1, textRange));
					final HighlightInfo.IntentionActionDescriptor newDescriptor2 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.ENUM, false),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor2, textRange));
					final HighlightInfo.IntentionActionDescriptor newDescriptor3 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.CLASS, true),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor3, textRange));
					final HighlightInfo.IntentionActionDescriptor newDescriptor4 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.INTERFACE, false),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor4, textRange));
					final HighlightInfo.IntentionActionDescriptor newDescriptor5 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.ANNOTATION_TYPE, false),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor5, textRange));

				}
				else if (buildCreateClassFromNewQuickFixes)
				{
					final TextRange textRange = element.getTextRange();
					final HighlightInfo.IntentionActionDescriptor newDescriptor1 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.CLASS, false),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor1, textRange));
					final HighlightInfo.IntentionActionDescriptor newDescriptor2 =
							new HighlightInfo.IntentionActionDescriptor(
									new LineageCreateClassFromUsageFix(SubclassType.CLASS, true),
									null, null);
					highlightInfo.quickFixActionRanges.add(new Pair(newDescriptor2, textRange));
				}
			}
		}
	}

	public void setRefCountHolder(RefCountHolder refCountHolder) {}

	public void init() {}

	public LineageHighlightVisitor clone()
	{
		return this;
	}
}