/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.intentions;

import com.intellij.psi.util.PsiElementFilter;
import com.intellij.psi.*;
import com.intellij.psi.tree.IElementType;
import net.intellij.plugins.lineage.tools.SubclassType;

class CreateClassFilter implements PsiElementFilter
{
	private final SubclassType subclassType;

	CreateClassFilter(SubclassType subclassType)
	{
		this.subclassType = subclassType;
	}

	public boolean isAccepted(PsiElement element)
	{
		if (!(element instanceof PsiJavaToken))
		{
			return false;
		}
		final PsiElement parent = element.getParent();
		if (!(parent instanceof PsiJavaCodeReferenceElement))
		{
			return false;
		}
		final PsiElement grandParent = parent.getParent();
		if (grandParent instanceof PsiNewExpression)
		{
			return subclassType == SubclassType.CLASS;
		}
		else if (grandParent instanceof PsiTypeElement)
		{
			return subclassType != SubclassType.ANNOTATION_TYPE;
		}
		else if (grandParent instanceof PsiReferenceList)
		{
			final PsiReferenceList referenceList = (PsiReferenceList) grandParent;
			final PsiElement firstChild = referenceList.getFirstChild();
			if (firstChild instanceof PsiKeyword)
			{
				final PsiKeyword keyword = (PsiKeyword) firstChild;
				final IElementType tokenType = keyword.getTokenType();
				if (JavaTokenType.IMPLEMENTS_KEYWORD.equals(tokenType))
				{
					return subclassType == SubclassType.INTERFACE;
				}
				else if (JavaTokenType.EXTENDS_KEYWORD.equals(tokenType) ||
						JavaTokenType.THROWS_KEYWORD.equals(tokenType))
				{
					return subclassType == SubclassType.CLASS;
				}
				else
				{
					return false;
				}
			}
			return false;

		}
		else if (grandParent instanceof PsiAnnotation)
		{
			return subclassType == SubclassType.ANNOTATION_TYPE;
		}
		else
		{
			return false;
		}
	}
}