/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.intentions;

import com.intellij.psi.PsiClass;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiJavaCodeReferenceElement;
import com.intellij.psi.PsiJavaToken;
import com.intellij.psi.util.PsiElementFilter;

class ImplementInterfaceFilter implements PsiElementFilter
{
	public boolean isAccepted(PsiElement element)
	{
		if (element == null)
		{
			return false;
		}
		final PsiElement parent = element.getParent();
		final PsiClass aClass;
		if (parent instanceof PsiClass)
		{
			aClass = (PsiClass)parent;
			final int offsetInParent = element.getStartOffsetInParent();
			final PsiJavaToken leftBrace = aClass.getLBrace();
			if (leftBrace == null)
			{
				return false;
			}
			final int leftBraceOffset = leftBrace.getStartOffsetInParent();
			if (offsetInParent >= leftBraceOffset)
			{
				return false;
			}
		}
		else if (parent instanceof PsiJavaCodeReferenceElement)
		{
			final PsiJavaCodeReferenceElement reference = (PsiJavaCodeReferenceElement)parent;
			final PsiElement resolvedElement = reference.resolve();
			if (!(resolvedElement instanceof PsiClass))
			{
				return false;
			}
			aClass = (PsiClass)resolvedElement;
		}
		else
		{
			return false;
		}
		return aClass.isInterface();
	}
}