/*
 * Copyright 2006 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.intentions;

import com.intellij.codeInsight.intention.IntentionAction;
import com.intellij.openapi.editor.CaretModel;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiFile;
import com.intellij.psi.util.PsiElementFilter;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.NotNull;

public abstract class LineageIntention implements IntentionAction
{
    private PsiElementFilter filter = null;
    @NotNull private String text = "<placeholder>";

	@Nullable
	protected final PsiElement findMatchingElement(PsiFile file, Editor editor)
	{
		final CaretModel caretModel = editor.getCaretModel();
		final int position = caretModel.getOffset();
		PsiElement element = file.findElementAt(position);
		while(element != null)
		{
			final PsiElementFilter filter = getCachedElementFilter();
			if(filter.isAccepted(element))
			{
				return element;
			}
			else
			{
				element = element.getParent();
			}
		}
		return null;
	}

    private PsiElementFilter getCachedElementFilter()
    {
        if (filter == null)
        {
            filter = getElementFilter();
        }
        return filter;
    }

    protected abstract PsiElementFilter getElementFilter();

    @NotNull
    public final String getText()
    {
        return text;
    }

    protected abstract String getTextForElement(PsiElement element);

    public final boolean isAvailable(Project project, Editor editor, PsiFile file)
    {
        final PsiElement element = findMatchingElement(file, editor);
        if(element != null)
        {
            text = getTextForElement(element);
        }
        return element != null;
    }
}
