/*
 * Copyright 2006-2007 Bas Leijdekkers
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.intellij.plugins.lineage.ui;

import org.jetbrains.annotations.Nullable;

import java.awt.Component;
import java.awt.Container;
import java.awt.FocusTraversalPolicy;
import java.util.ArrayList;
import java.util.List;

class ListFocusTraversalPolicy extends FocusTraversalPolicy
{

	private List<Component> componentList = new ArrayList();

	/**
	 * Add components in the order they should be traversed.
	 * @param component
	 */
	public void add(Component component)
	{
		componentList.add(component);
	}

	@Nullable
	public Component getComponentAfter(Container aContainer, Component aComponent)
	{
		int index = componentList.indexOf(aComponent);
		while (index < 0)
		{
			aComponent = aComponent.getParent();
			if (aComponent == null)
			{
				return null;
			}
			index = componentList.indexOf(aComponent);
		}
		if (index == componentList.size() - 1)
		{
			return null;
		}
		return componentList.get(index + 1);
	}

	@Nullable
	public Component getComponentBefore(Container aContainer, Component aComponent)
	{
		int index = componentList.indexOf(aComponent);
		while (index <= 0)
		{
			aComponent = aComponent.getParent();
			if (aComponent == null)
			{
				return null;
			}
			index = componentList.indexOf(aComponent);
		}
		return componentList.get(index - 1);
	}

	@Nullable
	public Component getFirstComponent(Container aContainer)
	{
		if (componentList == null || componentList.isEmpty())
		{
			return null;
		}
		return componentList.get(0);
	}

	@Nullable
	public Component getLastComponent(Container aContainer)
	{
		if (componentList == null || componentList.isEmpty())
		{
			return null;
		}
		return componentList.get(componentList.size() - 1);
	}

	@Nullable
	public Component getDefaultComponent(Container aContainer)
	{
		return getFirstComponent(aContainer);
	}
}